/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JOTMComponent.java 5602 2010-10-01 15:49:31Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.component.jotm;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.transaction.SystemException;
import javax.transaction.TransactionManager;

import org.objectweb.jotm.Jotm;
import org.objectweb.jotm.TimerManager;
import org.ow2.easybeans.component.api.EZBComponentException;
import org.ow2.easybeans.component.itf.TMComponent;
import org.ow2.easybeans.transaction.JTransactionManager;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Creates and binds the transaction factory and usertransaction object.
 * @author Florent Benoit
 */
public class JOTMComponent implements TMComponent {

    /**
     * Default Transaction timeout.
     */
    private static final int DEFAULT_TIMEOUT = 60;

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(JOTMComponent.class);

    /**
     * Jotm object.
     */
    private Jotm jotm = null;

    /**
     * Transaction timeout (in seconds).
     */
    private int timeout = DEFAULT_TIMEOUT;

    /**
     * Init method.<br/> This method is called before the start method.
     * @throws EZBComponentException if the initialization has failed.
     */
    public void init() throws EZBComponentException {

    }

    /**
     * Start method.<br/> This method is called after the init method.
     * @throws EZBComponentException if the start has failed.
     */
    public void start() throws EZBComponentException {

        // Create Transaction manager
        try {
            this.jotm = new Jotm(true, false);
        } catch (NamingException e) {
            throw new EZBComponentException("Cannot init JOTM object", e);
        }

        // Set transaction timeout
        try {
            this.jotm.getTransactionManager().setTransactionTimeout(this.timeout);
        } catch (SystemException se) {
            throw new EZBComponentException("Cannot set Transaction Timeout", se);
        }

        // Bind it
        try {
            new InitialContext().rebind(JNDI_NAME, this.jotm.getTransactionManager());
        } catch (NamingException e) {
            throw new EZBComponentException("Cannot bind user transaction", e);
        }

        // Init the static Transaction manager.
        JTransactionManager.init();

        // info
        logger.info("Register {0} as transaction manager object", JNDI_NAME);
    }

    /**
     * Stop method.<br/> This method is called when component needs to be
     * stopped.
     * @throws EZBComponentException if the stop is failing.
     */
    public void stop() throws EZBComponentException {

        // Unbind user transaction object
        try {
            new InitialContext().unbind(JNDI_NAME);
        } catch (NamingException e) {
            throw new EZBComponentException("Cannot unbind user transaction", e);
        }

        // Stop timer
        TimerManager.stop(true);

        // Stop JOTM
        this.jotm.stop();

    }

    /**
     * Gets the transaction manager object.
     * @return instance of the transaction manager
     */
    public TransactionManager getTransactionManager() {
        if (this.jotm != null) {
            return this.jotm.getTransactionManager();
        } else {
            return null;
        }
    }

    /**
     * Set the Transaction Timeout.
     * @param timeout Timeout (in seconds)
     */
    public void setTimeout(final int timeout) {
        this.timeout = timeout;
    }
}
